program XMLtoHTM;

{$APPTYPE CONSOLE}

uses
  windows,
  SysUtils,
  uMain in 'uMain.pas',
  CSplashWindow in '..\..\ControlsCep\Source\CSplashWindow.pas';

resourcestring
  resHelp = '    xml  html.' + #13#10 +
            '     Delphi' +
            #13#10 +
            '(. Generate XML documentation)   .' + #13 +
            #10#13#10 +
            ' :' + #13#10 +
            'XMLtoHTM [ ] [/ []]' + #13#10 +
            '       , .: ' +
            'C:\*.xml' + #13#10 +
            #13#10 +
            ' :';

  resEnter = #13#10 +
              ' enter   ';

  StrResult = ': - %d;   - %d;  - %d';

  resErrorParam = '  /%s   ';
  StrErrorParamName = '  /%s';
  StrErrorParamText = '  %s';
  StrErrorVal = '   /%s';

  StrErrorSource = '  ';
  StrErrorFile = '  %s';

  resH = ' ';
  resT = '  .    ' + #13 +
        #10 + ' xml-';
  resO = ',     .  ' +
        #13#10 +
        '     ,    .';
  resE = '  .   ' + DefaultExt;
  resL = '   xml-    ';
  resB = '   .  ' + #13#10 +
        '      .';
type
  TKey = record
    Key: Char;
    Comment: string;
  end;
const
  Keys: array [0..5] of TKey = ((Key: 'h'; Comment: resH),
                                (Key: 't'; Comment: resT),
                                (Key: 'o'; Comment: resO),
                                (Key: 'e'; Comment: resE),
                                (Key: 'l'; Comment: resL),
                                (Key: 'b'; Comment: resB));

type
TEventObj = class
private
  fLastSourceFolder: string;
  fLastDestFolder: string;
  procedure FileTransformed(Sender: TGenerateHTML; SourceName, DestName: string; var Continue: boolean);
  procedure ErrorTransformed(Sender: TGenerateHTML; SourceName, ErrorClassName, ErrorText: string; var Continue: boolean);
end;

var HTML: TGenerateHTML;
    EventObj: TEventObj;
    SourceName, XSLTName, OutputFolder, Ext: string;
    InsertHREF, IsBath: boolean;
    i, j, p: integer;

  function CharToOEM(S: string): AnsiString;
  begin
    result := '';
    if S <> '' then
    begin
      setLength(Result, Length(S));
      Windows.CharToOem(PChar(S), PAnsiChar(result));
    end;
  end;

  procedure OutStr(S: string);
  begin
    Writeln(CharToOem(S));
  end;

  function GetKey(I: integer): integer;
  var S: string;
  begin
    result := -1;
    if (i < 1) or (i > ParamCount) then Exit;
    S := ParamStr(i);
    if (Length(S) = 2) and (S[1] = '/') then
    begin
      for result := Low(Keys) to High(Keys) do
        if UpCase(Keys[result].Key) = UpperCase(S[2]) then
          Exit;
      raise Exception.CreateFmt(StrErrorParamName, [S[2]]);
    end;
  end;

  function GetVal(var I: integer): string;
  begin
    if i >= ParamCount then
      raise Exception.CreateFmt(StrErrorVal, [GetKey(i)]);
    inc(i);
    result := ParamStr(i);
  end;

{ TEventObj }

procedure TEventObj.ErrorTransformed(Sender: TGenerateHTML; SourceName,
  ErrorClassName, ErrorText: string; var Continue: boolean);
var S: string;
begin
  if not IsBath then
  begin
    OutStr(SourceName);
    S := '  ' + ErrorClassName + ':';
    while Length(S) < 10 do S := S + ' ';
    S := S + ' ' + ErrorText;
    OutStr(S);
  end;
  Continue := True;
end;

procedure TEventObj.FileTransformed(Sender: TGenerateHTML; SourceName,
  DestName: string; var Continue: boolean);
begin
  if not IsBath then
  begin
    if fLastSourceFolder <> ExtractFileDir(SourceName) then
    begin
      OutStr('Source:');
      fLastSourceFolder := ExtractFileDir(SourceName);
      OutStr(fLastSourceFolder);
    end;
    if (fLastDestFolder <> ExtractFileDir(DestName)) and (Sender.OutputFolder <> '') then
    begin
      OutStr('Destination:');
      fLastDestFolder := ExtractFileDir(DestName);
      OutStr(fLastDestFolder);
    end;
    OutStr(ExtractFileName(SourceName) + ' -> ' + ExtractFileName(DestName));
  end;
end;

begin
  try
    HTML := TGenerateHTML.Create;
    try
      EventObj := TEventObj.Create;
      try
        SourceName := '';
        XSLTName := '';
        OutputFolder := '';
        Ext := '';
        InsertHREF := False;
        IsBath := False;
        i := 1;
        while i <= ParamCount do
        begin
          p := GetKey(i);
          case p of
            0: begin
              OutStr(resHelp);
              for J := Low(Keys) to High(Keys) - 1 do
                OutStr('/' + Keys[j].Key + ' - ' + Keys[j].Comment);
              OutStr(resEnter);
              Readln;
              Exit;
            end;
            1: XSLTName := GetVal(i);
            2: OutputFolder := GetVal(i);
            3: Ext := GetVal(i);
            4: InsertHREF := True;
            5: IsBath := True;
           -1: if SourceName = '' then
                 SourceName := ParamStr(i)
               else
                 raise Exception.CreateFMT(StrErrorParamText, [ParamStr(i)]);
          end;
          inc(i);
        end;

        if SourceName = '' then
          raise Exception.Create(StrErrorSource);
        if (OutputFolder <> '') and (not DirectoryExists(OutputFolder)) then
          raise Exception.CreateFmt(strNoFolder, [OutputFolder]);
        if (XSLTName <> '') and (not FileExists(XSLTName)) then
          raise Exception.CreateFmt(StrErrorFile, [XSLTName]);

        HTML.onFileTransformed := EventObj.FileTransformed;
        HTML.onErrorTransformed := EventObj.ErrorTransformed;
        HTML.Execute(SourceName,
                     XSLTName,
                     OutputFolder,
                     Ext,
                     InsertHREF);
        OutStr(Format(StrResult, [HTML.FileCount,
                                   HTML.TransformedCount,
                                   HTML.ErrorCount]));
      finally
        FreeAndNil(EventObj);
      end;
    finally
      FreeAndNil(HTML);
    end;
  except
    on E:Exception do
    begin
      Writeln(E.Classname, ': ', CharToOem(E.Message));
      OutStr(Format(resErrorParam, [Keys[0].Key]));
      Halt(1);
    end;
  end;
end.














