unit uMain;

interface
uses Windows, SysUtils, Classes, MSXML2_TLB, ActiveX, CepUtils;
{$I Directives.inc}
resourcestring
  strNoSCHEMA = '  ';
  strNoFileSchema = '   %s';
  strNoFolder = '  %s';
  strNoRoot = '    xml,    ';
const
  DefaultExt = 'html';
type
  TGenerateHTML = class;

  TonFileTransformed = procedure (Sender: TGenerateHTML; SourceName, DestName: string; var Continue: boolean) of object;
  TonErrorTransformed = procedure (Sender: TGenerateHTML; SourceName, ErrorClassName, ErrorText: string; var Continue: boolean) of object;

  TGenerateHTML = class
  private
    fChanged: boolean;
    fUpdateCount: integer;
    fXSLT: string;
    fOutputFolder: string;
    fInsertHREF: boolean;
    fXML: string;
    fExt: string;
    fDOC: IXMLDOMDocument3;
    fSCHEMA: IXMLDOMDocument3;
    fTemplate: IXSLTemplate;
    fProcessor: IXSLProcessor;
    fSCHEMAName: string;
    faXSLTName: string;
    fFileCount: integer;
    fFileIndex: integer;
    fErrorCount: integer;
    fTransformedCount: integer;
    fonFileTransformed: TonFileTransformed;
    fonErrorTransformed: TonErrorTransformed;
    procedure SetOutputFolder(const Value: string);
    procedure SetXSLT(const Value: string);
    procedure SetInsertHREF(const Value: boolean);
    procedure SetXML(const Value: string);
    procedure SetExt(const Value: string);
    procedure CreateDoc(var DOC: IXMLDOMDocument3);
    function GetDOC: IXMLDOMDocument3;
    procedure CreateSchema(Name: string);
    procedure FreeSchema;
    function CalcFile(const FileName: string; const FileTime: TFileTime;
      const Size: int64): boolean;
  protected
    procedure Changed; virtual;
    procedure DoChanged; virtual;
    property DOC: IXMLDOMDocument3 read GetDOC;
    function EnumFile(const FileName: string; const FileTime: TFileTime; const Size: int64): boolean;
    procedure FileTransformed(SourceName, DestName: string; var Continue: boolean); virtual;
    procedure ErrorTransformed(SourceName, ErrorClassName, ErrorText: string; var Continue: boolean); virtual;
  public
    constructor Create; virtual;
    destructor Destroy; override;

    procedure BeginUpdate;
    procedure EndUpdate;

    {$IFDEF DELPHI_2005_UP}{$REGION '  xml-'}{$ENDIF}
    ///	<summary>  xml-</summary>
    {$IFDEF DELPHI_2005_UP}{$ENDREGION}{$ENDIF}
    property XML: string read fXML write SetXML;
    {$IFDEF DELPHI_2005_UP}{$REGION '    '}{$ENDIF}
    ///	<summary>    </summary>
    {$IFDEF DELPHI_2005_UP}{$ENDREGION}{$ENDIF}
    property XSLT: string read fXSLT write SetXSLT;

    {$IFDEF DELPHI_2005_UP}{$REGION '  '}{$ENDIF}
    ///	<summary>  </summary>
    {$IFDEF DELPHI_2005_UP}{$ENDREGION}{$ENDIF}
    property Ext: string read fExt write SetExt;

    {$IFDEF DELPHI_2005_UP}{$REGION '      .   ,      xml-.'}{$ENDIF}
    ///	<summary>      .   ,     
    ///	xml-.</summary>
    ///	<seealso cref="TGenerateHTML.XML">XML</seealso>
    {$IFDEF DELPHI_2005_UP}{$ENDREGION}{$ENDIF}
    property OutputFolder: string read fOutputFolder write SetOutputFolder;

    {$IFDEF DELPHI_2005_UP}{$REGION ' True,    xml-       ,     '}{$ENDIF}
    ///	<summary> True,    xml-       ,   
    ///	 </summary>
    ///	<seealso cref="XML">  xml-</seealso>
    {$IFDEF DELPHI_2005_UP}{$ENDREGION}{$ENDIF}
    property InsertHREF: boolean read fInsertHREF write SetInsertHREF;

    procedure LoadXMLFromStream(Stream: TStream);
    procedure SaveXMLToStream(Stream: TStream);
    procedure LoadXMLFromFile(FileName: string = '');
    procedure SaveXMLToFile(FileName: string = '');

    {$IFDEF DELPHI_2005_UP}{$REGION '    '}{$ENDIF}
    ///	<summary>    </summary>
    ///	<param name="Stream">,    </param>
    {$IFDEF DELPHI_2005_UP}{$ENDREGION}{$ENDIF}
    procedure SaveToStream(Stream: TStream);
    function SaveToFile(FileName: string = ''): string;

    property FileCount: integer read fFileCount;
    property FileIndex: integer read fFileIndex;
    property ErrorCount: integer read fErrorCount;
    property TransformedCount: integer read fTransformedCount;
    procedure Execute(aSourceName, aXSLTName, aOutputFolder, aExt: string;
                      aInsertHREF: boolean);

    property onFileTransformed: TonFileTransformed read fonFileTransformed write fonFileTransformed;
    property onErrorTransformed: TonErrorTransformed read fonErrorTransformed write fonErrorTransformed;
  end;


implementation

{ TGenerateHTML }

constructor TGenerateHTML.Create;
begin
  inherited Create;
  fExt := DefaultExt;
  CoInitialize(nil);
end;

procedure TGenerateHTML.CreateDoc(var DOC: IXMLDOMDocument3);
begin
  if DOC = nil then
  begin
    DOC := CoDOMDocument60.Create;
  end;
  DOC.async := False;
end;

destructor TGenerateHTML.Destroy;
begin
  FreeSchema;
  fDOC := nil;
  inherited;
end;

procedure TGenerateHTML.BeginUpdate;
begin
  inc(fUpdateCount);
end;

procedure TGenerateHTML.EndUpdate;
begin
  if fUpdateCount > 0 then
  begin
    Dec(fUpdateCount);
    if (fUpdateCount = 0) and fChanged then Changed;
  end;
end;

function TGenerateHTML.EnumFile(const FileName: string;
  const FileTime: TFileTime; const Size: int64): boolean;
var NewFileName: string;
begin
  try
    NewFileName := '';
    BeginUpdate;
    try
      XML := FileName;
      LoadXMLFromFile;
      if faXSLTName <> '' then
        XSLT := faXSLTName;
    finally
      EndUpdate;
    end;
    if InsertHREF and (faXSLTName <> '') then SaveXMLToFile;
    if Ext <> '' then
    begin
      NewFileName := SaveToFile;
    end;
    result := True;
    inc(fFileIndex);
    FileTransformed(FileName, NewFileName, result);
  except
    on E: Exception do
    begin
      result := false;
      inc(fFileIndex);
      ErrorTransformed(FileName, E.ClassName, E.Message, result);
    end;
  end;
end;

function TGenerateHTML.CalcFile(const FileName: string;
  const FileTime: TFileTime; const Size: int64): boolean;
begin
  inc(fFileCount);
  result := True;
end;


procedure TGenerateHTML.Execute(aSourceName, aXSLTName, aOutputFolder,
  aExt: string; aInsertHREF: boolean);
begin
  fFileCount := 0;
  fFileIndex := 0;
  fErrorCount := 0;
  fTransformedCount := 0;
  if (aOutputFolder <> '') and (not DirectoryExists(aOutputFolder)) then
    raise Exception.CreateFMT(strNoFolder, [aOutputFolder]);
  EnumerateFiles(ExtractFileDir(aSourceName),
                 ExtractFileName(aSourceName),
                 faAnyFile,
                 CalcFile);
  if fFileCount > 0 then
  begin
    faXSLTName := aXSLTName;
    OutputFolder := aOutputFolder;
    Ext := aExt;
    InsertHREF := aInsertHREF;
    EnumerateFiles(ExtractFileDir(aSourceName),
                   ExtractFileName(aSourceName),
                   faAnyFile,
                   EnumFile);
  end;
end;

function TGenerateHTML.GetDOC: IXMLDOMDocument3;
begin
  if fDOC = nil then
    CreateDoc(fDOC);
  Result := fDOC;
end;

procedure TGenerateHTML.Changed;
begin
  fChanged := True;
  if fUpdateCount = 0 then
  begin
    DoChanged;
    fChanged := False;
  end;
end;

procedure TGenerateHTML.DoChanged;
begin

end;

procedure TGenerateHTML.SetExt(const Value: string);
var V: string;
begin
  V := Trim(Value);
  while (Length(V) > 0) and (V[1] = '.') do Delete(V, 1, 1);
  if fExt <> V then
  begin
    fExt := V;
    Changed;
  end;
end;

procedure TGenerateHTML.SetInsertHREF(const Value: boolean);
begin
  if fInsertHREF <> Value then
  begin
    fInsertHREF := Value;
    Changed;
  end;
end;

procedure TGenerateHTML.SetOutputFolder(const Value: string);
begin
  if fOutputFolder <> trim(Value) then
  begin
    fOutputFolder := trim(Value);
    Changed;
  end;
end;

procedure TGenerateHTML.SetXML(const Value: string);
begin
  if fXML <> trim(Value) then
  begin
    fXML := trim(Value);
    Changed;
  end;
end;

procedure TGenerateHTML.SetXSLT(const Value: string);
begin
  if fXSLT <> trim(Value) then
  begin
    fXSLT := trim(Value);
    Changed;
  end;
end;

const StrSheetName = 'xml-stylesheet';
      StrType = 'type';
      StrTypeValue = 'text/xsl';
      StrHref = 'href';

function GetInstrNode(DOC: IXMLDOMDocument3; var Name: string): IXMLDOMNode;
var Node: IXMLDOMNode;
    I: integer;
    S: WideString;
    Instr: IXMLDOMDocument3;
    InstrNode: IXMLDOMNode;
    InstrStream: TMemoryStream;
    InstrFound: boolean;
begin
  result := nil;
  Name := '';
  S := '';
  Node := DOC.firstChild;
  if Node = nil then
    raise Exception.Create(strNoRoot);
  while (Node <> nil) and (Result = nil) do
  begin
    if (Node.nodeType = NODE_PROCESSING_INSTRUCTION) and
       (Node.nodeName = StrSheetName) then
    begin
      Instr := CoDOMDocument60.Create;
      Instr.async := False;
      InstrStream := TMemoryStream.Create;
      try
        S := '<' + StrSheetName + ' ' + Node.nodeValue + ' />';
        InstrStream.Write(S[1], Length(S) * SizeOf(S[1]));
        InstrStream.Position := 0;
        Instr.load(TStreamAdapter.Create(InstrStream) as IStream);
      finally
        FreeAndNil(InstrStream);
      end;
      S := '';
      InstrNode := Instr.firstChild;
      if InstrNode <> nil then
      begin
        S := '';
        InstrFound := False;
        if InstrNode.attributes <> nil then
        begin
          for I := 0 to InstrNode.attributes.length - 1 do
          begin
            if (InstrNode.attributes[I].nodeName = StrType) and
               (InstrNode.attributes[I].nodeValue = StrTypeValue) then
              InstrFound := True;
            if InstrNode.attributes[I].nodeName = StrHref then
              S := InstrNode.attributes[I].nodeValue;
          end;
        end;
        if InstrFound then
        begin
          Result := Node;
          Name := S;
        end;
      end;
    end;
    Node := Node.nextSibling;
  end;
end;

procedure InsertXmlStylesheet(DOC: IXMLDOMDocument3; Name: string);
var Node: IXMLDOMNode;
    OldName: string;
    Instr: IXMLDOMProcessingInstruction;
begin
  if Name = '' then
    raise Exception.Create(strNoSCHEMA);
  Node := GetInstrNode(DOC, OldName);
  DOC.removeChild(Node);
  Instr := DOC.createProcessingInstruction(StrSheetName,
                                           StrType + '="' +
                                           StrTypeValue + '" ' +
                                           StrHref + '="' +
                                           Name + '"');
  DOC.insertBefore(Instr, DOC.documentElement);

  //DOC.appendChild(Instr);
end;

procedure TGenerateHTML.LoadXMLFromStream(Stream: TStream);
var Adapter: IStream;
begin
  Adapter := TStreamAdapter.Create(Stream) as IStream;
  DOC.load(Adapter);
  GetInstrNode(DOC, fXSLT);
end;

procedure TGenerateHTML.SaveXMLToStream(Stream: TStream);
var Adapter: IStream;
begin
  if fInsertHREF then
  begin
    InsertXmlStylesheet(DOC, fXSLT);
  end;
  Adapter := TStreamAdapter.Create(Stream) as IStream;
  DOC.save(Adapter);
end;

procedure TGenerateHTML.LoadXMLFromFile(FileName: string);
var Stream: TFileStream;
begin
  if FileName = '' then FileName := fXML;
  Stream := TFileStream.Create(FileName, fmOpenRead);
  try
    LoadXMLFromStream(Stream);
  finally
    FreeAndNil(Stream);
  end;
end;

procedure TGenerateHTML.SaveXMLToFile(FileName: String);
var Stream: TFileStream;
begin
  if FileName = '' then FileName := fXML;
  Stream := TFileStream.Create(FileName, fmCreate);
  try
    SaveXMLToStream(Stream);
  finally
    FreeAndNil(Stream);
  end;
end;

function IsRelativePath(const Path: string): Boolean;
var
  L: Integer;
begin
  L := Length(Path);
  Result := (L > 0) and (Path[1] <> PathDelim)
    {$IFDEF MSWINDOWS}and (L > 1) and (Path[2] <> ':'){$ENDIF MSWINDOWS};
end;


function GetAbsolutePath(const BaseName, DestName: string): string;
var OldDir: string;
begin
  if IsRelativePath(DestName) then
  begin
    OldDir := GetCurrentDir;
    try
      SetCurrentDir(ExtractFilePath(BaseName));
      result := ExpandFileName(DestName);
    finally
      SetCurrentDir(OldDir);
    end;
  end
  else
    result := DestName;
end;

procedure TGenerateHTML.FileTransformed(SourceName, DestName: string;
  var Continue: boolean);
begin
  inc(fTransformedCount);
  if @fonFileTransformed <> nil then
    fonFileTransformed(self, SourceName, DestName, Continue);
end;

procedure TGenerateHTML.ErrorTransformed(SourceName, ErrorClassName,
  ErrorText: string; var Continue: boolean);
begin
  inc(fErrorCount);
  if @fonErrorTransformed <> nil then
    fonErrorTransformed(self, SourceName, ErrorClassName, ErrorText, Continue);
end;

procedure TGenerateHTML.FreeSchema;
begin
  fSCHEMAName := '';
  fProcessor := nil;
  if fTemplate <> nil then
    fTemplate.stylesheet := nil;
  fTemplate := nil;
  fSCHEMA := nil;
end;

procedure TGenerateHTML.CreateSchema(Name: string);
var TmpStream: TFileStream;
    Adapter: IStream;
begin
  FreeSchema;
  fSCHEMA := CoFreeThreadedDOMDocument60.Create;
  fSCHEMA.async := false;
  TmpStream := TFileStream.Create(Name, fmOpenRead);
  try
    Adapter := TStreamAdapter.Create(TmpStream) as IStream;
    fSCHEMA.load(Adapter);
    fTemplate := CoXSLTemplate60.Create;
    fTemplate.stylesheet := fSCHEMA;
    fProcessor := fTemplate.createProcessor();
  finally
    FreeAndNil(TmpStream);
  end;
  fSCHEMAName := Name;
end;

function TGenerateHTML.SaveToFile(FileName: string): string;
var Stream: TFileStream;
begin
  result := '';
  if (FileName = '') and (fXML <> '') and (fExt <> '') then
  begin
    FileName := ChangeFileExt(fXML, '.' + fExt);
    if fOutputFolder <> '' then
      FileName := IncludeTrailingPathDelimiter(fOutputFolder) +
                  ExtractFileName(FileName);
  end;
  try
    Stream := TFileStream.Create(FileName, fmCreate);
    try
      SaveToStream(Stream);
    finally
      FreeAndNil(Stream);
    end;
    result := FileName;
  except
    DeleteFile(FileName);
    raise;
  end;
end;

procedure TGenerateHTML.SaveToStream(Stream: TStream);
var SchemaName: string;
    Adapter: IStream;
begin
  if fXSLT = '' then
    raise Exception.Create(strNoSCHEMA);
  if fInsertHREF then
  begin
    InsertXmlStylesheet(DOC, fXSLT);
  end;
  SchemaName := GetAbsolutePath(fXML, fXSLT);
  if (fSCHEMAName <> SchemaName) or (fSCHEMA = nil) then
  begin
    if not FileExists(SchemaName) then
      raise Exception.CreateFMT(strNoFileSchema, [SchemaName]);
    CreateSchema(SchemaName);
  end;
  if fProcessor <> nil then
  begin
    fProcessor.reset;
    Adapter := TStreamAdapter.Create(Stream) as IStream;
    try
      fProcessor.input := DOC;
      fProcessor.output := Adapter;
      fProcessor.transform();
      Adapter.Commit(0);
    finally
      Adapter := nil;
    end;
  end;
end;

end.
